//************************************************************************
//**
//** Project......: Control DLL for the Firmware USB AVR Si570 controler.
//**
//** Platform.....: WinXP
//**
//** Licence......: This software is freely available for non-commercial 
//**                use - i.e. for research and experimentation only!
//**
//** Programmer...: F.W. Krom, PE0FKO
//** 
//** Description..: Control the Si570 Freq. PLL chip over the USB port.
//**
//** History......: V1.0 01/06/2009: First release of PE0FKO.
//**
//**************************************************************************
//
#include "stdafx.h"
#include <math.h>
#include "lusb0_usb.h"	// LIBUSB-Win32
#include "UsbAvrCmd.h"

// The ExtIO & CFGSR are not using the DLL,
// They are using direct linking of the code.
#if defined(EXTIO) || defined(CFGSR)
#define SRDLL_INCLUDE	1
#endif

#define SRDLL_EXPORTS	1
#include "SRDLL.h"

#define	usbHandle		((usb_dev_handle*)srUsbHandle)

SRDLL_API	void*		srUsbHandle = NULL;
SRDLL_API	int			srUsbTimeout = 500;

static	srUsbInfo_t		srUsbInfo;

#ifndef SRDLL_INCLUDE
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved			)
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			srUsbTimeout = 500;
			srUsbHandle = NULL;
			usb_init();
			break;

		case DLL_PROCESS_DETACH:
			srClose();
			break;

		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
			break;

    }
    return TRUE;
}
#endif

typedef	unsigned __int8		uint8_t;
typedef	unsigned __int16	uint16_t;
typedef	unsigned __int32	uint32_t;

static int
usbControlMsgIN(int cmd, int value, int index, char* buffer, int buflen)
{
	int r = -1;

	if (usbHandle != NULL)
	{
		r = usb_control_msg(usbHandle,
			USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN,
			cmd,
			value, index, buffer, buflen, srUsbTimeout);
	}

	return r;
}

static int
usbControlMsgOUT(int cmd, int value, int index, char* buffer, int buflen)
{
	int r = -1;

	if (usbHandle != NULL)
	{
		r = usb_control_msg(usbHandle,
			USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT,
			cmd,
			value, index, buffer, buflen, srUsbTimeout);
	}

	return r;
}


static int  usbGetStringAscii(usb_dev_handle *dev, int index, int langid, TCHAR *buf, int buflen)
{
	char    buffer[256];
	int     rval, i;

    if((rval = usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, (USB_DT_STRING << 8) + index, langid, buffer, sizeof(buffer), 1000)) < 0)
		return rval;

    if(buffer[1] != USB_DT_STRING)
        return 0;

    if((unsigned char)buffer[0] < rval)
        rval = (unsigned char)buffer[0];

    rval /= 2;

    /* lossy conversion to ISO Latin1 */
	for(i=1;i<rval;i++)
	{
		if(i > buflen)  /* destination buffer overflow */
			break;
		buf[i-1] = buffer[2 * i];
		if(buffer[2 * i + 1] != 0)  /* outside of ISO Latin1 range */
			buf[i-1] = '?';
    }

    buf[i-1] = 0;
    
	return i-1;
}

static BOOL
testUsbString(int ID, TCHAR* pStr, int iLen, const TCHAR* pTest)
{
	*pStr = '\0';

	if (ID > 0)
	{
		usbGetStringAscii(usbHandle, ID, 0x0409, pStr, iLen);
	}

	if (pTest != NULL)
	{
//		if (strncmp(pTest, pStr, iLen) != 0)
		if (wcscmp(pTest, pStr) != 0)
		{
			return FALSE;
		}
	}

	return TRUE;
}

SRAPI(int, srGetString, (int ID, TCHAR* pStr, int iLen))
{
	if (usbHandle == NULL)
		return 0;

	return usbGetStringAscii(usbHandle, ID, 0x0409, pStr, iLen);
}

SRAPI(void*, srOpen, (int vid, int pid, const TCHAR* pManufacturer, const TCHAR* pProduct, const TCHAR* pSerialNumber, int iDevNum))
{
	struct usb_bus *bus;
	struct usb_device *dev;

	if (pManufacturer != NULL && pManufacturer[0] == '\0') pManufacturer = NULL;
	if (pProduct      != NULL && pProduct[0]      == '\0') pProduct = NULL;
	if (pSerialNumber != NULL && pSerialNumber[0] == '\0') pSerialNumber = NULL;

	usb_find_busses();		// find all busses
	usb_find_devices();		// find all connected devices

	for(bus = usb_get_busses(); bus; bus = bus->next) 
	{
		for(dev = bus->devices; dev; dev = dev->next) 
		{
			if(dev->descriptor.idVendor == vid && dev->descriptor.idProduct == pid)
			{
				if ((srUsbHandle = usb_open(dev)) != NULL)
				{
					if (iDevNum == -1 || iDevNum == dev->devnum)
					{

						if(	testUsbString(dev->descriptor.iManufacturer, srUsbInfo.Manufacturer, sizeof(srUsbInfo.Manufacturer), pManufacturer)
						&&	testUsbString(dev->descriptor.iProduct,	     srUsbInfo.Product,      sizeof(srUsbInfo.Product),      pProduct)
						&&	testUsbString(dev->descriptor.iSerialNumber, srUsbInfo.SerialNumber, sizeof(srUsbInfo.SerialNumber), pSerialNumber))
						{
							srUsbInfo.VID = vid;
							srUsbInfo.PID = pid;

							if(usb_set_configuration(usbHandle, 1) < 0)
							{
								srClose();
								return NULL;
							}

							if(usb_claim_interface(usbHandle, 0) < 0)
							{
								srClose();
								return NULL;
							}

							return usbHandle;
						}
					}

					srClose();
				}
			}
		}
	}

	return NULL;
}

SRAPI(void, srClose, ())
{
	if (usbHandle != NULL)
	{
		usb_release_interface(usbHandle, 0);
		usb_close(usbHandle);
	}
	srUsbHandle = NULL;
}

SRAPI(BOOL, srIsOpen, ())
{
	return srUsbHandle != NULL;
}

SRAPI(srUsbInfo_t*, srGetUsbInfo, ())
{
	return srUsbHandle != NULL ? &srUsbInfo : NULL;
}

SRAPI(BOOL, srGetVersion, (int* major, int* minor))
{
	int r;
	uint16_t iVersion = 0;
	uint8_t  bVersion = 0;

	// Get version of my pe0fko firmware
	r = usbControlMsgIN(CMD_GET_VERSION, 0xA55A, 0, (char*)&iVersion, sizeof(iVersion));
	if (r == 2)
	{
		if (iVersion != 0xA55A)
		{
			*major = (iVersion >> 8) & 0xFF;
			*minor = iVersion & 0xFF;
		}
		else
		{
			// Echo the number, must be old SAQ firmware?
			*major = 1;		//vDG8SAQ
			*minor = 4;
		}

		return TRUE;
	}

	// Get version of the new V2.0 SDR-Kit firmware
	r = usbControlMsgIN(0xFF, 0, 0, (char*)&bVersion, sizeof(bVersion));
	if (r == 1 && bVersion != 0xFF)
	{
		*major = (bVersion >> 4) & 0x0F;
		*minor = bVersion & 0x0F;
		return TRUE;
	}

	return FALSE;
}

/*
Command 0x0F:
-------------
Restart the board (done by Watchdog timer).

Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x0F
    value:           Don't care
    index:           Don't care
    bytes:           NULL
    size:            0
*/
SRAPI(BOOL, srReboot, ())
{
	int r;

	r = usbControlMsgIN(CMD_REBOOT, 0, 0, NULL, sizeof(0));

	return r == 0;
}

/*
Command 0x17:
-------------
Read the Filter cross over points and set one point.

This command can control 2 banks of filters.  Typically the first bank is the Rx and Tx BPF
used in the QSD and QSE stages.  The second bank is the Tx LPF between the PA and the antenna
output.  The # of crossover points of the two banks of filters can be different.  For example,
the first bank is usually 4 bands (160m, 80/40m, 30/17/20m, 15/12/10m) as in Softrock v6.3 and
v9.0.  The second bank is usually 6 bands but can go up to 7, 8, 12 or even 16!

The index is used to specify the particular filter crossover point.  The index of the 1st bank
starts from 0, and ends at the last crossover point.  For example, if there are 4 bands, then
there will be 3 crossover points: 0, 1, and 2.  Index 3 is used as a boolean flag, specifying
whether this filter bank is enabled or disabled (for automatic band pass filter ABPF function).

The index of the 2nd filter bank starts from 256, and ends at the last crossover point.  For
example, if there are 6 LPF's, then there will be 5 crossover points, with index 256, 257, 258,
259, and 260.  index 261 is used as a boolean flag, specifying whether this filter bank is
enabled or disabled (for automatic switching).  If "disabled", it usually means the filter is
set for "all pass" or bypassed.

The first call to this command should be used to find out how many crossover points there are.
Call with an index of 255 for the 1st filter bank, and an index of 256+255 for the 2nd filter
bank.

  filter_number_of_bytes = usbCtrlMsgIN(0x17, 0, 255, FilterCrossOver, sizeof(FilterCrossOver));

If there are 4 filters (3 crossover points), then the filter_number_of_bytes returned will be 8.
(Each crossover point is 2 bytes.  Thus there will be 6 bytes.  Following that another 2 bytes will
be for the boolean flag, making a total of 8).

If ther are 6 filters (5 crossover points), there filter_number_of_bytes returned will be 12.

Subsequent calls to this command can then be used to:

1.  set one of the filter crossover points by specifying the index
2.  enable/disable the filter bank by specifying the last index (for the boolean flag)
3.  read the cross over points only - all of them of one bank at once, by specifying 255 (1st bank)
    or 256+255 (2nd bank).

(Note that in actions 1 and 2 above, the cross over points are also read out after the completion
of the action.)

The data format of the crossover points is a 11.5 bits in MHz, that gives a resolution of 1/32 MHz.
The last data entry is a boolean flag to enable or disable the filter bank.

Code sample:
   uint16_t FilterCrossOver[16];        // allocate enough space for up to 16 filters
   unsigned int filter_number_of_bytes;

  // first find out how may cross over points there are for the 1st bank, use 255 for index
  filter_number_of_bytes = usbCtrlMsgIN(0x17, 0, 255, FilterCrossOver, sizeof(FilterCrossOver));

  // Specify filter cross over point for a softrock that divide the LO by 4!
  // And read the points back from the device in the last call.
  if (filter_number_of_bytes == 8)  // 3 crossover points and one flag, so set them up
  {
	FilterCrossOver[0] = 4.1 * 4.0 * (1<<5);
	FilterCrossOver[1] = 8.0 * 4.0 * (1<<5);
	FilterCrossOver[2] = 16. * 4.0 * (1<<5);
	FilterCrossOver[3] = true;        // Enable

	usbCtrlMsgIN(0x17, FilterCrossOver[0], 0, NULL, 0);
	usbCtrlMsgIN(0x17, FilterCrossOver[1], 1, NULL, 0);
	usbCtrlMsgIN(0x17, FilterCrossOver[2], 2, NULL, 0);
	usbCtrlMsgIN(0x17, FilterCrossOver[3], 3, FilterCrossOver, sizeof(FilterCrossOver));
  }


Parameters: Setting one of the points
   requesttype:    USB_ENDPOINT_IN
   request:         0x17
   value:           FilterCrossOver[i]   i being the index of the particular cross over point
   index:           index of the 'value' filter point.
   bytes:           Array of up to 16 16bits integers for the filter points.
   size:            filter_number_of_bytes

Parameters: Enable / disable the filter
   requesttype:    USB_ENDPOINT_IN
   request:         0x17
   value:           0 (disable) or 1 (enable)
   index:           index of 1 plus the last crossover point
   bytes:           Array of up to 16 16bits integers for the filter points.
   size:            filter_number_of_bytes
*/
SRAPI(BOOL, srGetFilterRXTable, (double* pTable, int* pTableLength, BOOL* pEnable))
{
	int r;
	uint16_t iTable[256];

	if (pTable == NULL || pTableLength == NULL || *pTableLength < 1 || pEnable == NULL)
		return FALSE;

	r = usbControlMsgIN(CMD_SET_FILTER, 0, 255, (char*)iTable, sizeof(iTable));
	if (r <= 1)
	{
		*pTableLength = 0;
		*pEnable = FALSE;
		return FALSE;
	}

	int length = r / sizeof(iTable[0]) - 1;

	// Last entry is the enable
	*pEnable = iTable[length] != 0;
	
	if (length > *pTableLength)
		length = *pTableLength;

	for(int i=0; i < length; ++i) 
	{
		pTable[i] = (double)iTable[i] / (1UL << 5);
		pTable[i] = floor(pTable[i] * 32.0 + 0.5) / 32.0;		// Round to 1/32MHz
	}

	*pTableLength = length;

	return TRUE;
}

SRAPI(BOOL, srSetFilterRXPoint, (double Freq, int Entry))
{
	int len;
	uint16_t iTable[256];
	uint16_t COP = (uint16_t)(Freq * (1UL << 5));

	if ((len = usbControlMsgIN(CMD_SET_FILTER, 0, 255, (char*)iTable, sizeof(iTable))) > 0)
	{
		if (Entry < (len/2) && usbControlMsgIN(CMD_SET_FILTER, COP, Entry, NULL, 0) >= 0)
			return TRUE;
	}
	return FALSE;
}

SRAPI(BOOL, srSetFilterRXEnable, (BOOL bEnable))
{
	int len;
	uint16_t iTable[256];

	if ((len = usbControlMsgIN(CMD_SET_FILTER, 0, 255, (char*)iTable, sizeof(iTable))) > 0)
	{
		if (usbControlMsgIN(CMD_SET_FILTER, bEnable, (len/2)-1, NULL, 0) >= 0)
			return TRUE;
	}
	return FALSE;
}

SRAPI(BOOL, srGetFilterTXTable, (double* pTable, int* pTableLength, BOOL* pEnable))
{
	int r;
	uint16_t iTable[256];

	if (pTable == NULL || pTableLength == NULL || *pTableLength < 1 || pEnable == NULL)
		return FALSE;

	r = usbControlMsgIN(CMD_SET_FILTER, 0, 255 | 0x0100, (char*)iTable, sizeof(iTable));
	if (r <= 1)
	{
		*pTableLength = 0;
		*pEnable = FALSE;
		return FALSE;
	}

	int length = r / sizeof(iTable[0]) - 1;

	// Last entry is the enable
	*pEnable = iTable[length] != 0;
	
	if (length > *pTableLength)
		length = *pTableLength;

	for(int i=0; i < length; ++i) {
		pTable[i] = (double)iTable[i] / (1UL << 5);
		pTable[i] = floor(pTable[i] * 32.0 + 0.5) / 32.0;		// Round to 1/32MHz
	}

	*pTableLength = length;

	return TRUE;
}

SRAPI(BOOL, srSetFilterTXPoint, (double Freq, int Entry))
{
	int len;
	uint16_t iTable[256];
	uint16_t COP = (uint16_t)(Freq * (1UL << 5));

	if ((len = usbControlMsgIN(CMD_SET_FILTER, 0, 255 | 0x0100, (char*)iTable, sizeof(iTable))) > 0)
	{
		if (Entry < (len/2) && usbControlMsgIN(CMD_SET_FILTER, COP, Entry | 0x0100, NULL, 0) >= 0)
			return TRUE;
	}
	return FALSE;
}

SRAPI(BOOL, srSetFilterTXEnable, (BOOL bEnable))
{
	int len;
	uint16_t iTable[256];

	if ((len = usbControlMsgIN(CMD_SET_FILTER, 0, 255 | 0x0100, (char*)iTable, sizeof(iTable))) > 0)
	{
		if (usbControlMsgIN(CMD_SET_FILTER, bEnable, ((len/2)-1) | 0x0100, NULL, 0) >= 0)
			return TRUE;
	}
	return FALSE;
}

/*
------------------------
------- INDEX
------------------------

Command 0x39:
-------------
Return the frequency subtract multiply values.


Default:    subtract = 0.0, multiply = 1.0

Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x39
    value:           Don't care
    index:           Don't care
    bytes:           pointer 2 * 32 bits integer
    size:            8

Code sample:
    uint32_t iSM[2];
	double sub, mul;

    r = usbCtrlMsgIN(0x39, 0, 0, (char *)iSM, sizeof(iSM));
    if (r != sizeof(iSM)) Error

	sub = (double)(int32_t)iSM[0] / (1UL << 21); // Signed value
	mul = (double)         iSM[1] / (1UL << 21);
*/
SRAPI(BOOL, srGetFreqSubMul, (int index, double* pSub, double* pMul))
{
	int r;
	uint32_t iSM[2];

	r = usbControlMsgIN(CMD_GET_LO_SM, 0, index, (char*)iSM, sizeof(iSM));

	if (r == sizeof(iSM))
	{
		if (pSub != NULL)
		{
			*pSub = (double)(__int32)iSM[0] / (1UL << 21);	// Signed value
			*pSub = floor(*pSub * 1e6) / 1e6;				// Round to 1Hz
		}
		if (pMul != NULL)
		{
			*pMul = (double)iSM[1] / (1UL << 21);
			*pMul = floor(*pMul * 1e6 + 0.5) / 1e6;			// Round to 1Hz
		}
	}

	return r == sizeof(iSM);
}

/*
------------------------
------- INDEX
------------------------
Command 0x31:
-------------
Write the frequency subtract, multiply value's to the eeprom and use it.

The real frequency is the input frequnecy minus the subtract value times the multiply value.
Si570_F = (Finput - subtract) * multiply


Default:    None

Parameters:
    requesttype:    USB_ENDPOINT_OUT
    request:         0x31
    value:           Don't care
    index:           Don't care
    bytes:           pointer 2 * 32 bits interger
    size:            8
*/
SRAPI(BOOL, srSetFreqSubMul, (int index, double sub, double mul))
{
	int r;
	uint32_t iSM[2];

	iSM[0] = (uint32_t)( sub * (1UL << 21) ); // Signed value
	iSM[1] = (uint32_t)( mul * (1UL << 21) );

	r = usbControlMsgOUT(CMD_SET_LO_SM, 0, index, (char*)&iSM, sizeof(iSM));

	return r == sizeof(iSM);
}

/*
Command 0x3A:
-------------
Return actual frequency of the device.
The frequency is formatted in MHz as a 11.21 bits value.

Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x3A
    value:           Don't care
    index:           Don't care
    bytes:           pointer 32 bits integer
    size:            4

Code sample:
    uint32_t iFreq;
    double   dFreq;
    r = usbCtrlMsgIN(0x3A, 0, 0, (char *)&iFreq, sizeof(iFreq));
    if (r == 4)
        dFreq = (double)iFreq / (1UL<<21);
*/
SRAPI(BOOL, srGetFreq, (double* pMHz))
{
	int r;
	uint32_t iFreq;

	r = usbControlMsgIN(CMD_GET_FREQ, 0, 0, (char*)&iFreq, sizeof(iFreq));

	if (r == sizeof(iFreq))
	{
		if (pMHz != NULL) {
			*pMHz = (double)iFreq / (1UL << 21);
			*pMHz = floor(*pMHz * 1e6 + 0.5) / 1e6;			// Round to 1Hz
		}
	}

	return r == sizeof(iFreq);
}

/*
Command 0x32:
-------------
Set the oscillator frequency by value. The frequency is formatted in MHz
as 11.21 bits value. 
The "automatic band pass filter selection", "smooth tune", "one side calibration" and
the "frequency subtract multiply" are all done in this function. (if anabled in the firmware)

Default:    None

Parameters:
    requesttype:    USB_ENDPOINT_OUT
    request:         0x32
    value:           Don't care
    index:           Don't care
    bytes:           pointer 32 bits integer
    size:            4

Code sample:
    uint32_t iFreq;
    double   dFreq;

    dFreq = 30.123456; // MHz
    iFreq = (uint32_t)( dFreq * (1UL << 21) )
    r = usbCtrlMsgOUT(0x32, 0, 0, (char *)&iFreq, sizeof(iFreq));
    if (r < 0) Error
*/
// i2cAddr only needed with SAQ version firmware.
SRAPI(BOOL, srSetFreq, (double MHz, int i2cAddr))
{
	int r;
	uint32_t iFreq;

	iFreq = (uint32_t)(MHz * (double)(1UL << 21) + 0.5);

	r = usbControlMsgOUT(CMD_SET_FREQ, 0x0700 + i2cAddr, 0, (char*)(&iFreq), sizeof(iFreq));

	return r == sizeof(iFreq);
}

/*
Command 0x3F:
-------------
Return the Si570 frequency control registers (reg 7 .. 12). If there are I2C errors
the return length is 0.

Default:    None

Parameters:
    requesttype:     USB_ENDPOINT_IN
    request:         0x3F
    value:           Don't care
    index:           Don't care
    bytes:           pointer 6 byte register array
    size:            6
*/
// i2cAddr only needed with SAQ version firmware.
SRAPI(BOOL, srGetFreqReg, (unsigned char reg[6], int i2cAddr, int index))
{
	int r;

	r = usbControlMsgIN(CMD_GET_SI570, 0x0700 + i2cAddr, index, (char*)&reg[0], 6);	//sizeof(reg));

	return r == 6; //sizeof(reg);
}

/*
Command 0x30:
-------------
Set the oscillator frequency by Si570 register. The real frequency will be 
calculated by the firmware and the called command 0x32

Default:    None

Parameters:
    requesttype:    USB_ENDPOINT_OUT
    request:         0x30
    value:           I2C Address (only for the DG8SAQ firmware), Don't care
    index:           7 (only for the DG8SAQ firmware), Don't care
    bytes:           pointer 48 bits register
    size:            6
*/
// i2cAddr only needed with SAQ version firmware.
SRAPI(BOOL, srSetFreqReg, (unsigned char reg[6], int i2cAddr))
{
	int r;

	r = usbControlMsgOUT(CMD_SET_FREQ_REG, 0x0700 + i2cAddr, 0, (char*)&reg[0], 6);	//sizeof(reg));

	return r == 6; //sizeof(reg);
}

/*
Command 0x3D:
-------------
Return device crystal frequency.
The frequency is formatted in MHz as a 8.24 bits value.

Default:    114.285 MHz

Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x3D
    value:           Don't care
    index:           Don't care
    bytes:           pointer 32 bits integer
    size:            4
*/
SRAPI(BOOL, srGetXtalFreq, (double* pMHz))
{
	int r;
	uint32_t ixtal;

	// Try the new firmware command
	r = usbControlMsgIN(CMD_GET_XTAL, 0, 0, (char*)&ixtal, sizeof(ixtal));
	if (r == sizeof(ixtal))
	{
		if (pMHz != NULL) {
			*pMHz = (double)ixtal / (1UL << 24);
			*pMHz = floor(*pMHz * 0.125e6 + 0.5) / 0.125e6;		// Round to 0.125Hz
		}
		return TRUE;
	}

	// Old V14 Firmware DG8SAQ!
	for (int i=0; i<4; ++i)
	{
		r = usbControlMsgIN(CMD_READ_EEPROM, 2+i, 0, &((char*)&ixtal)[i], 1);
		if (r != 1)
		{
			return FALSE;
		}
	}

	if (pMHz != NULL) {
		*pMHz = (double)ixtal / (1UL << 24);
		*pMHz = floor(*pMHz * 0.125e6 + 0.5) / 0.125e6;			// Round to 0.125Hz
	}

	return TRUE;
}

/*
Command 0x33:
-------------
Write new crystal frequency to EEPROM and use it. It can be changed to calibrate the device.
The frequency is formatted in MHz as a 8.24 bits value.

Default:    114.285 MHz

Parameters:
    requesttype:    USB_ENDPOINT_OUT
    request:         0x33
    value:           Don't care
    index:           Don't care
    bytes:           pointer 32 bits integer
    size:            4
*/
SRAPI(BOOL, srSetXtalFreq, (double MHz))
{
	int r;
	uint32_t iFreq;
	
	iFreq = (uint32_t)(MHz * (1UL << 24) + 0.5);

	r = usbControlMsgOUT(CMD_SET_XTAL, 0, 0, (char*)&iFreq, sizeof(iFreq));

	return r == sizeof(iFreq);
}

/*
Command 0x3C:
-------------
Return device startup frequency.
The frequency is formatted in MHz as a 11.21 bits value.

Default:    4 * 7.050 MHz

Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x3C
    value:           Don't care
    index:           Don't care
    bytes:           pointer 32 bits integer
    size:            4
*/
SRAPI(BOOL, srGetStartupFreq, (double* pMHz))
{
	int r;
	uint32_t iFreq;

	r = usbControlMsgIN(CMD_GET_STARTUP, 0, 0, (char*)(&iFreq), sizeof(iFreq));

	if (r == sizeof(iFreq))
	{
		if (pMHz != NULL) {
			*pMHz = (double)iFreq / (1UL << 21);
			*pMHz = floor(*pMHz * 1e6 + 0.5) / 1e6;			// Round to 1Hz
		}
	}

	return r == sizeof(iFreq);
}

/*
Command 0x34:
-------------
Write new startup frequency to eeprom. When the device is started it will output
this frequency until a program set an other frequency.
The frequency is formatted in MHz as a 11.21 bits value.

Default:    4 * 7.050 MHz

Parameters:
    requesttype:    USB_ENDPOINT_OUT
    request:         0x34
    value:           Don't care
    index:           Don't care
    bytes:           pointer 32 bits integer
    size:            4
*/
SRAPI(BOOL, srSetStartupFreq, (double MHz))
{
	int r;
	uint32_t iFreq;

	iFreq = (uint32_t)(MHz * (1UL << 21) + 0.5);

	r = usbControlMsgOUT(CMD_SET_STARTUP, 0, 0, (char*)&iFreq, sizeof(iFreq));

	return r == sizeof(iFreq);
}

/*
Command 0x3B:
-------------
Return the "Smooth tune" PPM (pulse per MHz) of the device.
The value is default 3500 (from data sheet) and can be changed. I do not know what 
happened with the chip if it is out of range (>3500).
If the value is set to zero it will disable the "Automatic Smooth tune" function.

Default:    3500 PPM

Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x3B
    value:           Don't care
    index:           Don't care
    bytes:           pointer 16 bits integer
    size:            2
*/
SRAPI(BOOL, srGetSmoodTuneRange, (int* pRange))
{
	int r;
	uint16_t iRange;

	r = usbControlMsgIN(CMD_GET_PPM, 0, 0, (char*)&iRange, sizeof(iRange));

	if (r == sizeof(iRange))
	{
		if (pRange != NULL)
			*pRange = iRange;
	}

	return r == sizeof(iRange);
}

/*
Command 0x35:
-------------
Write new smooth tune to eeprom and use it.

Default:    3500 PPM

Parameters:
    requesttype:    USB_ENDPOINT_OUT
    request:         0x35
    value:           Don't care
    index:           Don't care
    bytes:           pointer 16 bits integer
    size:            2
*/
SRAPI(BOOL, srSetSmoodTuneRange, (int Range))
{
	int r;
	uint16_t iRange;

	iRange = Range;

	r = usbControlMsgOUT(CMD_SET_PPM, 0, 0, (char*)&iRange, sizeof(iRange));

	return r == sizeof(iRange);
}

/*
Command 0x41:
-------------
Set a new I2C address for the Si570 chip.
The function can also be used to reset the device to "factory default" by writing the
value 255. After a restart the device will initialize to all the default values.

Default:    0x55 (85 decimal)

Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x41
    value:           I2C address or 255
    index:           0
    bytes:           NULL
    size:            0
*/
SRAPI(BOOL, srSetI2CAddr, (int i2c))
{
	int r;
	uint8_t iI2C;

	if (i2c != 0xff)
		i2c &= 0x7f;

	if (i2c == 0)
		return FALSE;

	r = usbControlMsgIN(CMD_SET_I2C_ADDR, i2c, 0, (char*)&iI2C, sizeof(iI2C));
	if (r == sizeof(iI2C) && iI2C != 0xFF)
		return TRUE;

	return FALSE;
}

SRAPI(BOOL, srGetI2CAddr, (int* pI2C))
{
	int r;
	uint8_t iI2C;

	r = usbControlMsgIN(CMD_SET_I2C_ADDR, 0, 0, (char*)&iI2C, sizeof(iI2C));
	if (r == sizeof(iI2C) && iI2C != 0xFF)
	{
		if (pI2C != NULL)
			*pI2C = iI2C;

		return TRUE;
	}

	return FALSE;
}

SRAPI(BOOL, srSetUsbId, (char Id))
{
	int r;
	uint8_t iId;

	if (!isprint(Id))
		return FALSE;

	r = usbControlMsgIN(CMD_GET_USB_ID, Id, 0, (char*)&iId, sizeof(iId));
	if (r == sizeof(iId) && iId != 0xFF)
		return TRUE;

	return FALSE;
}

SRAPI(BOOL, srGetUsbId, (char* pId))
{
	int r;
	uint8_t iId;

	r = usbControlMsgIN(CMD_GET_USB_ID, 0, 0, (char*)&iId, sizeof(iId));
	if (r == sizeof(iId) && iId != 0xFF)
	{
		if (pId != NULL)
			*pId = iId;

		return TRUE;
	}

	return FALSE;
}

/*
Command 0x50:
-------------
Set PTT (PB4) I/O line and read CW key level from the PB5 (CW Key_1) and PB1 (CW Key_2).
In case of the enabled ABPF no change of PTT I/O line will be done and no read of the CW key's are
done. The command will return (in case of enabled ABPF) for both CW key's a open status (bits are 1).
The returnd bit value is bit 5 (0x20) for CW key_1 and bit 1 (0x02) for CW key_2, the other bits are zero.


Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x50
    value:           Output BOOL to user output PTT
    index:           0
    bytes:           pointer to 1 byte variable CW Key's
    size:            1
*/
SRAPI(BOOL, srSetPTTGetCWInp, (int ptt, int* pCWkey))
{
	int r;
	uint8_t key;

	r = usbControlMsgIN(CMD_SET_PTT, ptt, 0, (char*)&key, sizeof(key));
	if (r < 0)
		return FALSE;

	if (r == sizeof(key) && pCWkey != NULL)
		*pCWkey = key;

	return TRUE;
}

/*

Command 0x51:
-------------
Read CW key level from the PB5 (CW Key_1) and PB1 (CW Key_2).
In case of the enabled ABPF no read of the CW key's are done. The command will return for both CW key's 
a open status (bits are 1).
The returnd bit value is bit 5 (0x20) for CW key_1 and bit 1 (0x02) for CW key_2, the other bits are zero.


Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x51
    value:           0
    index:           0
    bytes:           pointer to 1 byte variable CW Key's
    size:            1
*/
SRAPI(BOOL, srGetCWInp, (int* pCWkey))
{
	int r;
	uint8_t key;

	r = usbControlMsgIN(CMD_GET_CW_KEY, 0, 0, (char*)&key, sizeof(key));

	if (r == sizeof(key))
	{
		if (pCWkey != NULL)
			*pCWkey = key;
	}

	return r == sizeof(key);
}

/*
Command 0x20:
-------------
Write one byte to a Si570 register. Return value is the i2c error boolean in the buffer array.

Code sample:
	// Si570 RECALL function
	uint8_t i2cError;
    r = usbCtrlMsgIN(0x20, 0x55 | (135<<8), 0x01, &i2cError, 1);
	if (r == 1 && i2cError == 0)
		// OK

Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x20
    value:           I2C Address low byte (only for the DG8SAQ firmware)
                     Si570 register high byte
    index:           Register value low byte
    bytes:           NULL
    size:            0
*/
SRAPI(BOOL, srSetRegSi570, (int reg, int value, int i2cAddr, BOOL* pI2CError))
{
	int r;
	uint8_t iI2CError;

	r = usbControlMsgIN(CMD_SET_SI570, (reg << 8) + (i2cAddr & 0xFF), value, (char*)&iI2CError, sizeof(iI2CError));

	if (pI2CError != NULL)
		*pI2CError = iI2CError != 0;

	return r == sizeof(iI2CError);
}

/*
Command 0x15:
-------------
Set the I/O bits of the device. The SoftRock V9 only had two I/O lines, bit0 and bit1.
It also returned the I/O value (like command 0x16).

There are two values for every I/O bit, the data direction and data bits.
+-----+------+----------------------
| DDR | DATA | PIN Function
+-----+------+----------------------
|  0  |   0  | input
|  0  |   1  | input internal pullup
|  1  |   0  | output 0
|  1  |   1  | output 1
+-----+------+----------------------

In case of the enabled ABPF no change of I/O will be done.

Code sample:
    uint16_t INP;
    r = usbCtrlMsgIN(0x15, 0x02, 0x02, &INP, sizeof(INP));
    // Set P2 to output and one!
    // Use P1 as input, no internal pull up R enabled.
    // Read the input in array INP[], only bit0 and bit1 used by this hardware.

Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x15
    value:           Data Direction Register
    index:           Data register
    bytes:           PIN status (returned)
    size:            2
*/
SRAPI(BOOL, srSetIO, (int IOdat, int IOddr))
{
	int r;

	r = usbControlMsgIN(CMD_SET_IO, IOddr, IOdat, NULL, 0);

	return r == 0;
}

/*
Command 0x16:
-------------
Get the I/O values in the returned word, the SoftRock V9 only had two I/O lines, bit0 and bit1.

Code sample:
    uint16_t INP;
    r = usbCtrlMsgIN(0x16, 0, 0, &INP, sizeof(INP));
    // Read the input word INP, only bit0 and bit1 used by SoftRock V9 hardware.

Parameters:
    requesttype:    USB_ENDPOINT_IN
    request:         0x16
    value:           Don't care
    index:           Don't care
    bytes:           PIN status (returned)
    size:            2
*/
SRAPI(BOOL, srGetIO, (int* pIOval))
{
	int r;
	uint16_t io;

	r = usbControlMsgIN(CMD_GET_IO, 0, 0, (char*)&io, sizeof(io));
	if (r == sizeof(io))
	{
		if (pIOval != NULL)
			*pIOval = io;
	}

	return r == sizeof(io);
}

/*
Command 0x42:
-------------
Get the cpu chip temperature

Index 0 = Temperature in degC.Signed Int.  0 = 0 deg C
                 0x7ff0 =  127.9375 deg C (128*0x7ff0/0x8000), 0xfff0 = -128 deg C
*/
SRAPI(BOOL, srGetCpuTemp, (double* pTemp))
{
	int r;
	uint16_t iTemp;

	r = usbControlMsgIN(CMD_GET_CPU_TEMP, 0, 0, (char*)&iTemp, sizeof(iTemp));
	if (r == sizeof(iTemp))
	{
		if (pTemp != NULL)
		{
			*pTemp = (double)iTemp / (1UL << 4);
		}
	}

	return r == sizeof(iTemp);
}


/*
Convert the Si570 registers back to the frequency. 
The (calibrated) crystal frequency is needed in that calculation.
*/
SRAPI(double, srFromSi570RegToFreq, (unsigned char reg[6], double xtal))
{
	int N1, HS_DIV;
	union { __int8 b[8]; __int64 qw; } RFREQ;
	double dRFREQ;

	HS_DIV = (reg[0] >> 5) & 0x07;
	N1 = (reg[0] & 0x1F) << 2 | (reg[1] >> 6) & 0x03;
	RFREQ.qw = 0;
	RFREQ.b[0] = reg[5];
	RFREQ.b[1] = reg[4];
	RFREQ.b[2] = reg[3];
	RFREQ.b[3] = reg[2];
	RFREQ.b[4] = reg[1] & 0x3F;

	dRFREQ = (double)RFREQ.qw / (1UL << 28);

	return xtal * dRFREQ / ((HS_DIV+4) * (N1+1));
}



// Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete
/*
Command 0x01:
-------------
Set port directions.
Do not use, use I/O function 0x15
*/
SRAPI(BOOL, srSetPortDDR, (int DDR))
{
	int r;

	r = usbControlMsgIN(CMD_SET_DDR, DDR, 0, NULL, 0);

	return r == 0;
}


// Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete
/*
Command 0x02:
-------------
Read ports.
Do not use, use I/O function 0x15
*/
SRAPI(BOOL, srGetPortPIN, (int* pPort))
{
	int r;
	uint8_t iPort;

	r = usbControlMsgIN(CMD_GET_PIN, 0, 0, (char*)&iPort, sizeof(iPort));

	if (r == sizeof(iPort))
	{
		if (pPort != NULL)
			*pPort = iPort;
	}

	return r == sizeof(iPort);
}


// Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete
/*
Command 0x03:
-------------
Read port states.
Do not use.
*/
SRAPI(BOOL, srGetPortPRT, (int* pPort))
{
	int r;
	uint8_t iPort;

	r = usbControlMsgIN(CMD_GET_PORT, 0, 0, (char*)&iPort, sizeof(iPort));

	if (r == sizeof(iPort))
	{
		if (pPort != NULL)
			*pPort = iPort;
	}

	return r == sizeof(iPort);
}

// Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete Obsolete
/*
Command 0x04:
-------------
Set ports.
In case of the enabled ABPF no change of I/O will be done.
Do not use, use I/O function 0x15
*/
SRAPI(BOOL, srSetPort, (int Port))
{
	int r;

	r = usbControlMsgIN(CMD_SET_PORT, Port, 0, NULL, 0);

	return r == 0;
}

SRAPI(BOOL, srGetBandRXFilters, (int* length, int* Filters))
{
	int r;
	uint8_t iFilters[100];

	r = usbControlMsgIN(CMD_GET_RX_BAND_FILTER, 0, 0, (char*)iFilters, sizeof(iFilters));
	if (r > 1)	// r==1 data=0xff Unknow command!
	{
		*length = r;

		for(int i=0; i<r; ++i)
			Filters[i] = iFilters[i];
	}

	return r > 1;
}

SRAPI(BOOL, srSetBandRXFilter, (int band, int filter))
{
	int r;
	uint8_t iFilters[100];

	r = usbControlMsgIN(CMD_SET_RX_BAND_FILTER, filter, band, (char*)iFilters, sizeof(iFilters));

	return r > 1;
}

SRAPI(BOOL, srGetBandTXFilters, (int* length, int* Filters))
{
	int r;
	uint8_t iFilters[10];

	r = usbControlMsgIN(CMD_GET_TX_BAND_FILTER, 0, 0, (char*)iFilters, sizeof(iFilters));
	if (r > 1)	// r==1 data=0xff Unknow command!
	{
		*length = r;

		for(int i=0; i<r; ++i)
			Filters[i] = iFilters[i];
	}

	return r > 1;
}

SRAPI(BOOL, srSetBandTXFilter, (int band, int filter))
{
	int r;

	r = usbControlMsgIN(CMD_SET_TX_BAND_FILTER, filter, band, NULL, 0);

	return r == 0;
}

//++ Mobo extensions

SRAPI(BOOL, srMoboSetPABias, (int iIndex, int iValue, int* pOldValue))
{
	int r;
	uint8_t iOldValue;

	r = usbControlMsgIN(CMD_RM_PA_BIAS, iValue, iIndex, (char*)&iOldValue, sizeof(iOldValue));
	if (r != 1 || iOldValue == 0xFF)
		return FALSE;

	if (pOldValue != NULL)
		*pOldValue = iOldValue;

	return TRUE;
}


SRAPI(BOOL, srMoboGetAdc, (int iIndex, double* pValue, double dMax))
{
	int r;
	uint16_t iValue;

	r = usbControlMsgIN(CMD_GET_ADC_INPUTS, 0, iIndex, (char*)&iValue, sizeof(iValue));
	if (r != sizeof(iValue))
		return FALSE;

	if (pValue != NULL)
	{
		*pValue = dMax * iValue / 65536.0;
	}

	return TRUE;
}


SRAPI(BOOL, srGetSi570Grade, (int* pGrade, int* pDCOmin, int* pDCOmax, int* piRFFreqIndex, BOOL* pbFreezeM))
{
	int r;
	struct {
		uint16_t	dcomin;
		uint16_t	dcomax;
		uint8_t		grade;
		uint8_t		index;
	} iValue;

	r = usbControlMsgIN(CMD_SET_SI570_GRADE, 0, 0, (char*)&iValue, sizeof(iValue));
//	if (r != 5)	//sizeof(iValue)) is 6, aligment
	if (r != 5 && r != 6)	// V15.15
		return FALSE;

	if (pGrade != NULL)
		*pGrade = iValue.grade;

	if (pDCOmin != NULL)
		*pDCOmin = iValue.dcomin;

	if (pDCOmax != NULL)
		*pDCOmax = iValue.dcomax;

	if (piRFFreqIndex != NULL && pbFreezeM != NULL) {
		*piRFFreqIndex = 0;
		*pbFreezeM = FALSE;
		if (r == 6)
		{
			*piRFFreqIndex = iValue.index & 0x7F;
			*pbFreezeM = iValue.index & 0x80 ? TRUE : FALSE;
		}
	}

	return TRUE;
}

SRAPI(BOOL,	srSetSi570Grade, (int iGrade, int iDCOmin, int iDCOmax, int iRfFreqIndex, BOOL bFreezeM))
{
	int r;
	struct {
		uint16_t	dcomin;
		uint16_t	dcomax;
		uint8_t		grade;
		uint8_t		index;
	} iValue;

	if (iGrade > 0)
	{
		UINT tValue;
		tValue = iGrade | (iRfFreqIndex << 8) | (bFreezeM ? 0x8000 : 0);

		r = usbControlMsgIN(CMD_SET_SI570_GRADE, tValue, 0, (char*)&iValue, sizeof(iValue));
		if (r != 5 && r != 6)	// V15.14 --> r == 5; V15.15 --> r == 6
			return FALSE;
	}

	if (iDCOmin > 0)
	{
		r = usbControlMsgIN(CMD_SET_SI570_GRADE, 0x0000, iDCOmin, (char*)&iValue, sizeof(iValue));
		if (r != 5 && r != 6)	// V15.14 --> r == 5; V15.15 --> r == 6
			return FALSE;
	}

	if (iDCOmax > 0)
	{
		r = usbControlMsgIN(CMD_SET_SI570_GRADE, 0x0100, iDCOmax, (char*)&iValue, sizeof(iValue));
		if (r != 5 && r != 6)	// V15.14 --> r == 5; V15.15 --> r == 6
			return FALSE;
	}

	return TRUE;
}


/**
static double rfpowercalc(double voltage)
{
	double power;

	if (voltage > 0.0)
		voltage += 0.1;		// 100mV Schottky diode loss

	power = voltage * 1000 / 102.5;
	power = (power * power) / 50.0;

	return power;
}

static double SWRcalc(double Vfwd, double Vref)
{
	if (Vfwd < 0.001)
		return 1.0;

	if (Vref > Vfwd)
		return 99.9;

	double swr = (Vfwd + Vref) / (Vfwd - Vref);

	return (swr > 99.9) ? 99.9 : swr;
}

SRAPI(BOOL, srMoboGetAdcPACurrent, (double* pCurrent))
{
	return MoboGetAdc(0, pCurrent, 2.5);
}

SRAPI(BOOL, srMoboGetAdcPAPowerOutput, (double* pPower)	// Power in Wath)
{
	if (!MoboGetAdc(1, pPower, (double)0xfff0 / 1000.0))
		return FALSE;

	if (pPower != NULL)
	{
		*pPower = rfpowercalc(*pPower);
	}

	return TRUE;
}

SRAPI(BOOL, srMoboGetAdcPAPowerReflected, (double* pPower)	// Power in Wath)
{
	if (!MoboGetAdc(2, pPower, (double)0xfff0 / 1000.0))
		return FALSE;

	if (pPower != NULL)
	{
		*pPower = rfpowercalc(*pPower);
	}

	return TRUE;
}

SRAPI(BOOL, srMoboGetAdcPASWR, (double* pSWR))
{
	double Vfwd, Vref;

	if (!MoboGetAdc(1, &Vfwd, 5.0) || !MoboGetAdc(2, &Vref, 5.0))
		return FALSE;

	if (pSWR != NULL)
	{
		*pSWR = SWRcalc(Vfwd, Vref);
	}

	return TRUE;
}

SRAPI(BOOL, srMoboGetAdcPAVoltage, (double* pVoltage))
{
	return MoboGetAdc(3, pVoltage, 15.64);
}

SRAPI(BOOL, srMoboGetAdcPATemperature, (double* pTemp))
{
	BOOL err = MoboGetAdc(4, pTemp, 128.0 * 2.0);
	if (*pTemp >= 128.0)
		*pTemp = 127.9375 - *pTemp;
	return err;
}
/*
Index 0 = PA current, full scale 0xfff0 = 2.5A
Index 1 = Power Output
Index 2 = Power reflected
Index 3 = Supply voltage, full scale 0xfff0 = 15.64V
Index 4 = Temperature in degC.Signed Int.  0 = 0 deg C
         0x7ff0 =  127.9375 deg C (128*0x7ff0/0x8000), 0xfff0 = -128 deg C

		 0x0000	0
		 0x7ff0 128
		 0xfff0	-128

		 0x0000	0A
		 0xfff0	2.5A

*/
//-- Mobo extensions
